#!/usr/bin/perl -w
# $Id: bootstrap_chromosome.pl,v 1.10 2010/07/07 12:30:58 rnl14480 Exp $

use strict;
use lib qw(../lib/);

use BAOBAB::ChromosomeWrapper;
use BAOBAB::Chromosome;
use BAOBAB::OrganismWrapper;
use BAOBAB::Organism;
use BAOBAB::Fasta;
use BAOBAB::Fasta::Sequence;
use BAOBAB::Wrapper;
use DBI;

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

die "Usage: $0 data-dir\n" unless scalar(@ARGV) >= 1;

my $data = shift;

my @organisms =
  qw (random_ada random_aga random_planted_metazoa random_dmel ada aga planted_metazoa dmel);

foreach my $org (@organisms) {

	print "Processing organism $org...\n";
	my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );
	$ow->read( shortname => $org );

	my $handler  = get_handler($org);
	my $filename = get_filename($org);

	print "Creating FASTA object for file $filename\n";
	my $fasta =
	  BAOBAB::Fasta->new( filename => $filename, header_handler => $handler );
	foreach my $seq ( $fasta->sequences ) {
		my ( $name, $len ) = ( $seq->id, $seq->length );
		print "\tSequence $name ($len nt)\n";
		my $c = BAOBAB::Chromosome->new(
			organism => $ow->wrap,
			name     => $name,
			size     => $len
		);
		my $cw = BAOBAB::ChromosomeWrapper->new( object => $c, dbh => $dbh );
		$cw->write;
	}
}
$dbh->disconnect();

sub get_filename {

	my $o    = shift;
	my %data = (
		ada                    => $data . q(darlingi.nfa),
		aga                    => $data . q(aga.nfa),
		planted_metazoa        => $data . q(planted_metazoa.nfa),
		random_ada             => $data . q(random_ada.nfa),
		random_aga             => $data . q(random_aga.nfa),
		random_planted_metazoa => $data . q(random_planted_metazoa.nfa),
		random_dmel            => $data . q(random_dmel.nfa),
		dmel                   => $data . q(dmel.nfa)
	);

	return $data{$o};
}

sub get_handler {

	my $o    = shift;
	my %data = (
		ada => sub { $_ = shift; s/\s//g; return $_; },
		aga                    => sub { ( split /:/, shift )[1] },
		planted_metazoa        => sub { return 'U' },
		random_ada             => sub { return 'U' },
		random_aga             => sub { return 'U' },
		random_planted_metazoa => sub { return 'U' },
		random_dmel            => sub { return 'U' },
		dmel                   => sub {
			$_ = shift;
			if (/^arm_(\S+)/) {
				return $1;
			}
			elsif (/^Arm(\S+)/) {
				return $1;
			}
			elsif (/^(\S+)/) {
				return $1;
			}
			else {
				return '??';
			}
		}
	);

	return $data{$o};
}
